package uy.edu.fing.tse.central.db.dao.fact;

import uy.edu.fing.tse.central.db.entity.Calificacion;
import uy.edu.fing.tse.central.db.entity.Categoria;
import uy.edu.fing.tse.central.db.entity.Estado;
import uy.edu.fing.tse.central.db.entity.Hecho;
import uy.edu.fing.tse.central.db.mapper.MyMapper;
import uy.edu.fing.tse.dto.Fact;

import javax.annotation.PostConstruct;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Stateless
public class FactDAOBean implements FactDaoLocal {

    @PersistenceContext(unitName = "central")
    private EntityManager em;

    @PostConstruct
    void init() {
        //Despues del constructor hace esto
    }

    @Override
    public Fact create(Fact elem) {
        final var hecho = MyMapper.INSTANCE.convert(elem);
        {
            final var query = em.createNamedQuery("Estado.findByValue", Estado.class);
            query.setParameter("val", elem.getActualState().getValue());
            final var e = query.getSingleResult();
            hecho.setActualState(e);
        }

        if (elem.getScore() != null) {
            final var query = em.createNamedQuery("Calificacion.findByValue", Calificacion.class);
            query.setParameter("val", elem.getScore().getValue());
            final var s = query.getSingleResult();
            hecho.setScore(s);
        }

        if (elem.getCategory() != null) {
            final var query = em.createNamedQuery("Categoria.findByName", Categoria.class);
            query.setParameter("name", elem.getCategory().getName());
            final var c = query.getSingleResult();
            hecho.setCategory(c);
        }

        em.persist(hecho);
        em.flush();

        return MyMapper.INSTANCE.convert(hecho);
    }

    @Override
    public Fact find(String key) {
        return find(Long.parseLong(key));
    }

    @Override
    public Fact find(Long id) {
        final var hecho = em.find(Hecho.class, id);
        return MyMapper.INSTANCE.convert(hecho);
    }

    @Override
    public List<Fact> findAll() {
        final var query = em.createNamedQuery("Hecho.findAll", Hecho.class);
        final List<Hecho> hechos = query.getResultList();
        return hechos.stream().map(MyMapper.INSTANCE::convert).collect(Collectors.toList());
    }

    @Override
    public List<Fact> search(final String value, final String state, final Date after, final Date before) {
        boolean addWhere = true;
        Map<String, Object> params = new HashMap<>();
        String hql = "SELECT DISTINCT h FROM Hecho h INNER JOIN h.actualState AS e INNER JOIN h.history AS s ";

        if (notNullOrEmpty(value)) {
            hql += " WHERE ";
            addWhere = false;
            hql += "( h.title LIKE %:title%";
            hql += " OR ";
            hql += " h.desc LIKE %:description% )";
            params.put("title", value);
            params.put("description", value);
        }

        if (notNullOrEmpty(state)) {
            if (addWhere) {
                hql += " WHERE ";
                addWhere = false;
            } else {
                hql += " AND ";
            }
            hql += " e.value = :state";
            params.put("state", state);
        }

        if (after != null) {
            if (addWhere) {
                hql += " WHERE ";
                addWhere = false;
            } else {
                hql += " AND ";
            }
            hql += " s.date <= :after";
            params.put("after", after);
        }

        if (before != null) {
            if (addWhere) {
                hql += " WHERE ";
                addWhere = false;
            } else {
                hql += " AND ";
            }
            hql += " s.date >= :before";
            params.put("state", before);
        }

        final var query = em.createQuery(hql, Hecho.class);
        for (Map.Entry<String, Object> param : params.entrySet()) {
            query.setParameter(param.getKey(), param.getValue());
        }
        final List<Hecho> hechos = query.getResultList();
        return hechos.stream().map(MyMapper.INSTANCE::convert).collect(Collectors.toList());
    }

    @Override
    public List<Fact> filter(final String title, final String desc, final String state) {
        boolean addWhere = true;
        Map<String, String> params = new HashMap<>();
        String hql = "SELECT h FROM Hecho h INNER JOIN h.actualState AS e ";

        if (notNullOrEmpty(title)) {
            hql += " WHERE ";
            addWhere = false;
            hql += " h.title = :title";
            params.put("title", title);
        }
        if (notNullOrEmpty(desc)) {
            if (addWhere) {
                hql += " WHERE ";
                addWhere = false;
            } else {
                hql += " AND ";
            }
            hql += " h.desc = :description";
            params.put("description", desc);
        }
        if (notNullOrEmpty(state)) {
            if (addWhere) {
                hql += " WHERE ";
                addWhere = false;
            } else {
                hql += " AND ";
            }
            hql += " e.value = :state";
            params.put("state", state);
        }
        final var query = em.createQuery(hql, Hecho.class);
        for (Map.Entry<String, String> param : params.entrySet()) {
            query.setParameter(param.getKey(), param.getValue());
        }
        final List<Hecho> hechos = query.getResultList();
        return hechos.stream().map(MyMapper.INSTANCE::convert).collect(Collectors.toList());
    }

    private boolean nullOrEmpty(final String val) {
        return val == null || val.isEmpty();
    }

    private boolean notNullOrEmpty(final String val) {
        return !nullOrEmpty(val);
    }
}
